"""
pip install pystray Pillow requests bs4 lxml pywin32 win11toast
"""

from pathlib import Path

import win32con
import win32console
import win32gui
from PIL import Image
from bs4 import BeautifulSoup
from pystray import Icon, MenuItem, Menu
from requests import get
from win11toast import toast


def get_text(url: str) -> (str, bool):
    """
    Получение кода страницы по заданной ссылке.
    """
    headers = {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                      'Chrome/110.0.0.0 Safari/537.36'
    }
    try:
        res = get(url=url, headers=headers)
        return res.text if res.status_code == 200 else False
    except Exception:
        return False


def parse_course(url: str) -> (str, bool):
    """
    Парсинг стоимости валюты по отношению к рублю.
    """
    try:
        if res := get_text(url):
            soup = BeautifulSoup(res, 'lxml')
            course = soup.find('div', class_='dDoNo ikb4Bb gsrt').find('span', class_='DFlfde SwHCTb').text.strip()
            return course if course else False
        else:
            return False
    except AttributeError:
        return False


def notify_send(text: str, url='', icon=None):
    """
    Вывод уведомления с активными кнопками.
    """
    if icon:
        buttons = [{'activationType': 'protocol', 'arguments': f'{url}', 'content': 'Открыть в Google'},
                   {'activationType': 'protocol', 'arguments': 'http:Dismiss', 'content': 'Закрыть'}]
        toast("Курс к рублю", f'{text}', buttons=buttons, icon=icon)
    else:
        toast("Курс к рублю", f'{text}',
              button={'activationType': 'protocol', 'arguments': 'http:Dismiss', 'content': 'Закрыть'}, icon=icon)


def notify_choice(course: str, url: str, val: str):
    """
    Выбор сообщения для определенной валюты, в зависимости от значения val.
    """
    if val == "b":
        icon = {
            'src': f'{Path.cwd() / "pic" / "free-dollar.png"}',
            'placement': 'appLogoOverride'
        }
        notify_send(f'Курс ₽1/$1 составляет: {course} руб.за доллар', url, icon)
    elif val == "y":
        icon = {
            'src': f'{Path.cwd() / "pic" / "free-yen.png"}',
            'placement': 'appLogoOverride'
        }
        notify_send(f'Курс ₽1/JP¥1 составляет: {course} руб.за йену', url, icon)
    elif val == "u":
        icon = {
            'src': f'{Path.cwd() / "pic" / "free-yuan.png"}',
            'placement': 'appLogoOverride'
        }
        notify_send(f'Курс ₽1/CN¥1 составляет: {course} руб.за юань', url, icon)


def click(icon: Icon, item: MenuItem):
    """
    Обработка полученных значений меню. В зависимости от выбранного
    пункта выполняется запрос данных по определенной валюте.
    """
    if str(item) == 'доллара':
        url = 'https://www.google.ru/search?q=курс+доллара'
        if course := parse_course(url):
            notify_choice(course, url, "b")
        else:
            notify_send("Данные по курсу не получены")
    elif str(item) == 'йены':
        url = 'https://www.google.ru/search?q=курс+йены'
        if course := parse_course(url):
            notify_choice(course, url, "y")
        else:
            notify_send("Данные по курсу не получены")
    elif str(item) == 'юаня':
        url = 'https://www.google.ru/search?q=курс+юаня'
        if course := parse_course(url):
            notify_choice(course, url, "u")
        else:
            notify_send("Данные по курсу не получены")
    elif str(item) == 'Выход':
        icon.stop()


def main():
    image = Image.open(Path.cwd() / "pic" / "cat.png")
    icon = Icon('Cat Vampire', image, menu=Menu(
        MenuItem('Курс к рублю:', Menu(
            MenuItem('доллара', click),
            MenuItem('йены', click),
            MenuItem('юаня', click)
        )),
        MenuItem('Выход', click)
    ))
    win32gui.ShowWindow(win32console.GetConsoleWindow(), win32con.SW_HIDE)
    icon.run()


if __name__ == "__main__":
    main()
